import json

from .config import get_store_reader


def get_cache_filename():
    return f".{get_store_reader()('ENV')}-cache.json"


def cache_read():
    try:
        with open(get_cache_filename(), 'r') as f:
            return json.load(f)
    except FileNotFoundError:
        return {}
    except json.JSONDecodeError:
        return {}


def cache_write(cache):
    with open(get_cache_filename(), 'w') as f:
        json.dump(cache, f, indent=2)


def cache_set(key, value):
    cache = cache_read()
    cache[key] = value
    cache_write(cache)
    return value


def cache_get(key):
    cache = cache_read()
    return cache.get(key, None)


def retrieve_and_cache(retrieve_func):
    def with_arguments(args=None):
        def with_key(key):
            def with_describer(describe_func):
                resp = cache_get(key)
                if resp is None:
                    print(f"Cache miss key <{key}>")
                    if args is None:
                        retrieve_result = retrieve_func()
                    else:
                        retrieve_result = retrieve_func(args)
                    cache_set(key, retrieve_result)
                else:
                    print(f"Cache hit key <{key}>")
                resp = cache_get(key)
                print(f"{key}: {describe_func(resp)}")
                return resp
            return with_describer
        return with_key
    return with_arguments



def create_memoizer(func):
    func_result_cache = {}

    def memoize(arg=None):
        cached_value = func_result_cache.get(arg, None)
        if cached_value is None:
            if arg is None:
                func_result_cache[arg] = func()
            else:
                func_result_cache[arg] = func(arg)
        return func_result_cache.get(arg)

    return memoize



import os

from dotenv import dotenv_values

from .memoize import create_memoizer


def config():
    environment_name = os.environ.get('ENV', 'dev')
    os.environ['ENV'] = environment_name
    values = {
        **dotenv_values(f".{environment_name}.env"),
        **os.environ
    }
    return values


memoize = create_memoizer(config)


def get_store_reader():
    store = memoize()

    def get_env(key):
        return store.get(key, None)

    return get_env


def get_store_writer():
    store = memoize()

    def get_env_key_writer(key):
        def set_env_key(value):
            store[key] = value

        return set_env_key

    return get_env_key_writer



def date_diff(a_date):
    def with_date(date_to_subtract):
        difference = a_date - date_to_subtract

        def apply(func=None):
            if func is None:
                return difference
            return func(difference)
        return apply
    return with_date


def in_minutes():
    def apply_to(difference):
        minutes = difference.seconds // 60
        if difference.days < 0:
            minutes = minutes + difference.days * 24 * 60

        def with_func(func=None):
            if func is None:
                return minutes
            return func(minutes)
        return with_func
    return apply_to
